function pairinds = gibbsgenepairs(params, paramsigma, expr, npoints, nsteps, foldchangelim)

% generate pairs of rnaseq points which represent one gene either side of
% the developmental transition

% need to specify the allowed steps in parameter space
% need to supply a threshold value of expression change in order for the 
% point to be considered

% the function works by doing the following
% - randomly select the first point
% - assign a probability to every point based on the distance from the
%   chosen point (using paramsigma to determine how likely the step is)
% - set to zero any points which fall outside the foldchangelim
%   requirements (actually do this first!)
% - randomly choose one of the remaining points, weighted by its
%   probability

% for Gibbs sampling, alternate between first and second point, resampling
% each time, and recording at defined intervals

if nargin<6 || isempty(foldchangelim)
    foldchangelim = [-Inf,Inf];
end

if nargin<5 || isempty(nsteps)
    % the number of steps between each recorded sample
    nsteps = 1000;
end

pairinds = zeros(npoints,2);

keep = 0;
while(nnz(keep)==0)
    % start off by sampling the end point
    ind1 = ceil(npoints * rand);
    
    % select by foldchange range
    keep = expr<(expr(ind1)/foldchangelim(1)) & expr>(expr(ind1)/foldchangelim(2));
end
    
wb = timerBar('Generating pairs',npoints);

for count = 1:(npoints * nsteps);
    % given the current end point, resample the start point
    
    
    kinds = find(keep); % could modify the probability based on the fold change rather than straight 0 or 1
    prep = zeros(numel(kinds),1);
    for jj = 1:size(params,2)
        prep = prep - ((params(kinds,jj) - params(ind1,jj)).^2)/(2*paramsigma(jj)^2);
    end
    pvals = exp(prep);

    cp = cumsum(pvals);
    subind = nnz((cp(1:(end-1))/cp(end))<rand) + 1;
    ind0 = kinds(subind);
    
    
    % now resample the end point
    keep = expr>(expr(ind0)*foldchangelim(1)) & expr<(expr(ind0)*foldchangelim(2));
    
    % Gaussian probability
    kinds = find(keep);
    prep = zeros(numel(kinds),1);
    for jj = 1:size(params,2)
        prep = prep - ((params(kinds,jj) - params(ind0,jj)).^2)/(2*paramsigma(jj)^2);
    end
    pvals = exp(prep);

    cp = cumsum(pvals);
    subind = nnz((cp(1:(end-1))/cp(end))<rand) + 1;
    ind1 = kinds(subind);
    
    % update keep for next round
    keep = expr<(expr(ind1)/foldchangelim(1)) & expr>(expr(ind1)/foldchangelim(2));
    
    % check for recording
    if mod(count,nsteps)==0
        
        pairinds(count/nsteps,:) = [ind0,ind1];
        
        wb.increment();
        % could also sample the next set of random numbers here if we
        % want..
    end
    
end

wb.finish();
